@@ -3,7 +3,9 @@ Main class for trace operations
 """
 from flask import g, request
 import requests
-from flask_trace_util.util import extract_gcloud_trace
+
+from opencensus.trace.tracer import Tracer
+from opencensus.trace.samplers import AlwaysOnSampler
 
 
 class _FlaskTrace:
@@ -11,8 +13,6 @@ class _FlaskTrace:
         self.app = None
         self.extractor = kwargs.get("extractor")
         self.delegator = kwargs.get("delegator")
-        self.user_id_extractor = kwargs.get("user_id_extractor")
-        self.user_id_delegator = kwargs.get("user_id_delegator")
 
     def init_app(self, app, **kwargs):
         """
@@ -21,12 +21,6 @@ class _FlaskTrace:
         self.app = app
         self.extractor = kwargs.get("extractor") or self.extractor
         self.delegator = kwargs.get("delegator") or self.delegator
-        self.user_id_extractor = (
-            kwargs.get("user_id_extractor") or self.user_id_extractor
-        )
-        self.user_id_delegator = (
-            kwargs.get("user_id_delegator") or self.user_id_delegator
-        )
 
         if self.app is None:
             raise Exception("app cant be None")
@@ -39,15 +33,17 @@ class _FlaskTrace:
 
         @app.before_request
         def init_session():  # pylint: disable=unused-variable
-            g.trace = self.extractor()
-            if self.user_id_extractor:
-                g.user_id = self.user_id_extractor()
+            trace_context = self.extractor()
+            if "tracer" in trace_context:
+                trace_context["tracer"].start_span(name=request.url)
+            g.trace_context = trace_context
 
         @app.after_request
         def destroy_session(
             response,
         ):  # pylint: disable=unused-variable,unused-argument
-            g.trace = None
+            if g.trace_context and "tracer" in g.trace_context:
+                g.trace_context["tracer"].tracer.finish()
             return response
 
     def request(self, method, url, **kwargs):
@@ -62,46 +58,13 @@ class _FlaskTrace:
         """
         if "headers" not in kwargs:
             kwargs["headers"] = {}
-        header_name, header_value = self.delegator()
-        kwargs["headers"][header_name] = header_value
-        if self.user_id_delegator:
-            u_header_name, u_header_value = self.user_id_delegator()
-            kwargs["headers"][u_header_name] = u_header_value
+        headers = self.delegator()
+        for header_name, header_value in headers.items():
+            kwargs["headers"][header_name] = header_value
         return requests.request(method, url, **kwargs)
 
 
-def gcloud_trace_extractor(project_id):
-    """Trace extractor for google cloud GKE
-
-    Arguments:
-        project_id {str} -- project id
-    """
-
-    def extractor():
-        trace = request.headers.get("X-Cloud-Trace-Context")
-        _, trace = extract_gcloud_trace(trace, project_id)
-        return trace
-
-    return extractor
-
-
-def gcloud_trace_delegator():
-    """Trace delegator for google cloud GKE
-
-    Arguments:
-        trace {str} -- trace
-    """
-
-    def delegator():
-        trace = ""
-        if g.trace:
-            trace = g.trace.split("/")[-1]
-        return "X-Cloud-Trace-Context", trace
-
-    return delegator
-
-
-def simple_user_id_extractor():
+def user_id_extractor():
     """Simple user_id extractor
     """
 
@@ -109,19 +72,64 @@ def simple_user_id_extractor():
         user_id = request.headers.get("X-User-Id")
         if not user_id:
             user_id = "Anonymous"
-        return user_id
+        return "user_id", user_id
 
     return extractor
 
 
-def simple_user_id_delegator():
+def user_id_delegator():
     """Simple user_id delegator
     """
 
     def delegator():
         user_id = "Anonymous"
-        if g.user_id:
-            user_id = g.user_id
+        if g.trace_context and "user_id" in g.trace_context:
+            user_id = g.trace_context["user_id"]
         return "X-User-Id", user_id
 
     return delegator
+
+
+def opencencus_trace_extractor(exporter, header, propogator):
+    def extractor():
+        trace = request.headers.get(header)
+        span_context = propogator.from_header(trace)
+        tracer = Tracer(
+            exporter=exporter, span_context=span_context, sampler=AlwaysOnSampler(),
+        )
+        return "tracer", tracer
+
+    return extractor
+
+
+def opencencus_trace_delegator(header, propogator):
+    def delagator():
+        trace = ""
+        if g.trace_context and "tracer" in g.trace_context:
+            span_context = g.trace_context["tracer"].span_context
+            trace = propogator.to_header(span_context)
+        return header, trace
+
+    return delagator
+
+
+def trace_extractor(extractors):
+    def extractor():
+        trace_context = {}
+        for ex in extractors:
+            k, val = ex()
+            trace_context[k] = val
+        return trace_context
+
+    return extractor
+
+
+def trace_delegator(delegators):
+    def delegator():
+        headers = {}
+        for dele in delegators:
+            k, val = dele()
+            headers[k] = val
+        return headers
+
+    return delegator
